from __future__ import annotations

from abc import abstractmethod
from collections.abc import Iterable
from typing import Any

from pycep.typing import ResourceAttributes

from checkov.bicep.checks.resource.registry import registry
from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class BaseResourceCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: "Iterable[CheckCategories]",
        supported_resources: "Iterable[str]",
        guideline: str | None = None,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type="resource",
            guideline=guideline,
        )
        self.supported_resources = supported_resources
        registry.register(self)

    def scan_entity_conf(self, conf: ResourceAttributes, entity_type: str) -> CheckResult:  # type:ignore[override]  # it's ok
        if conf["existing"] is True:
            # the existing keyword is used to retrieve information about an already deployed resource
            return CheckResult.UNKNOWN

        self.entity_type = entity_type
        self.api_version = conf["api_version"]

        return self.scan_resource_conf(conf["config"])

    @abstractmethod
    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        raise NotImplementedError()



from __future__ import annotations

import csv
import os
from datetime import datetime
from typing import Any

from checkov.common.output.report import Report, CheckType

date_now = f'{datetime.now().strftime("%Y%m%d-%H%M%S")}'
FILE_NAME_OSS_PACKAGES = f'{date_now}_oss_packages.csv'
HEADER_OSS_PACKAGES = ["Package", "Version", "Path", "git org", "git repository", "Vulnerability", "Severity",
                       "License"]
HEADER_CONTAINER_IMAGE = HEADER_OSS_PACKAGES
FILE_NAME_CONTAINER_IMAGES = f'{date_now}_container_images.csv'

FILE_NAME_IAC = f'{date_now}_iac.csv'
HEADER_IAC = ["Resource", "Path", "git org", "git repository", "Misconfigurations", "Severity"]

CTA_NO_API_KEY = "SCA, image and runtime findings are only available with Bridgecrew. Signup at " \
                 "https://www.bridgecrew.cloud/login/signUp and add your API key to include those findings. "


class CSVSBOM():

    def __init__(self) -> None:
        self.iac_rows: list[dict[str, Any]] = []
        self.container_rows: list[dict[str, Any]] = []
        self.package_rows: list[dict[str, Any]] = []

    def add_report(self, report: Report, git_org: str, git_repository: str) -> None:
        if report.check_type != CheckType.SCA_IMAGE and report.check_type != CheckType.SCA_PACKAGE:
            for failed_record in report.failed_checks:
                self.iac_rows.append(
                    {"Resource": failed_record.resource, "Path": failed_record.file_path, "git org": git_org,
                     "git repository": git_repository, "Misconfigurations": failed_record.check_id,
                     "Severity": failed_record.severity})
            for extra_resource in report.extra_resources:
                self.iac_rows.append(
                    {"Resource": extra_resource.resource, "Path": extra_resource.file_path, "git org": git_org,
                     "git repository": git_repository, "Misconfigurations": "",
                     "Severity": ""})

    def persist_report(self, is_api_key: bool) -> None:
        CSVSBOM.write_section(file=FILE_NAME_IAC, header=HEADER_IAC, rows=self.iac_rows,
                              is_api_key=True)
        CSVSBOM.write_section(file=FILE_NAME_CONTAINER_IMAGES, header=HEADER_CONTAINER_IMAGE, rows=self.container_rows,
                              is_api_key=is_api_key)
        CSVSBOM.write_section(file=FILE_NAME_OSS_PACKAGES, header=HEADER_OSS_PACKAGES, rows=self.package_rows,
                              is_api_key=is_api_key)

    @staticmethod
    def write_section(file: str, header: list[str], rows: list[dict[str, Any]], is_api_key: bool) -> None:
        with open(file, 'w', newline='') as f:
            print(f'Persisting SBOM to {os.path.abspath(file)}')
            if is_api_key:
                dict_writer = csv.DictWriter(f, fieldnames=header)
                dict_writer.writeheader()
                dict_writer.writerows(rows)
            else:
                writer = csv.writer(f)
                writer.writerow(header)
                writer.writerow([CTA_NO_API_KEY])



from __future__ import annotations

import itertools
import os
import sys
from typing import TYPE_CHECKING

from cyclonedx.model import XsUri, ExternalReference, ExternalReferenceType, sha1sum, HashAlgorithm, HashType
from cyclonedx.model.bom import Bom, Tool
from cyclonedx.model.component import Component, ComponentType
from cyclonedx.model.vulnerability import Vulnerability, VulnerabilityAdvisory, BomTarget
from cyclonedx.output import SchemaVersion, get_instance
from packageurl import PackageURL  # type:ignore[import]

if sys.version_info >= (3, 8):
    from importlib.metadata import version as meta_version
else:
    from importlib_metadata import version as meta_version

if TYPE_CHECKING:
    from checkov.common.output.extra_resource import ExtraResource
    from checkov.common.output.record import Record
    from checkov.common.output.report import Report

DEFAULT_CYCLONE_SCHEMA_VERSION = SchemaVersion.V1_4
CYCLONE_SCHEMA_VERSION: dict[str, SchemaVersion] = {
    "1.4": DEFAULT_CYCLONE_SCHEMA_VERSION,
    "1.3": SchemaVersion.V1_3,
    "1.2": SchemaVersion.V1_2,
    "1.1": SchemaVersion.V1_1,
    "1.0": SchemaVersion.V1_0,
}


class CycloneDX:
    def __init__(self, reports: list[Report]) -> None:
        self.reports = reports

        self.bom = self.create_bom()

    def create_bom(self) -> Bom:
        bom = Bom()

        try:
            version = meta_version("checkov")  # type:ignore[no-untyped-call]  # issue between Python versions
        except Exception:
            # Unable to determine current version of 'checkov'
            version = "UNKNOWN"

        this_tool = Tool(vendor="bridgecrew", name="checkov", version=version)
        this_tool.external_references.update(
            [
                ExternalReference(
                    reference_type=ExternalReferenceType.BUILD_SYSTEM,
                    url=XsUri("https://github.com/bridgecrewio/checkov/actions"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.DISTRIBUTION,
                    url=XsUri("https://pypi.org/project/checkov/"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.DOCUMENTATION,
                    url=XsUri("https://www.checkov.io/1.Welcome/What%20is%20Checkov.html"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.ISSUE_TRACKER,
                    url=XsUri("https://github.com/bridgecrewio/checkov/issues"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.LICENSE,
                    url=XsUri("https://github.com/bridgecrewio/checkov/blob/master/LICENSE"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.SOCIAL,
                    url=XsUri("https://twitter.com/bridgecrewio"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.VCS,
                    url=XsUri("https://github.com/bridgecrewio/checkov"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.WEBSITE,
                    url=XsUri("https://www.checkov.io/"),
                ),
            ]
        )

        bom.metadata.tools.add(this_tool)

        for report in self.reports:
            for check in itertools.chain(report.passed_checks, report.skipped_checks):
                component = self.create_component(check_type=report.check_type, resource=check)

                if not bom.has_component(component=component):
                    bom.components.add(component)

            for check in report.failed_checks:
                component = self.create_component(check_type=report.check_type, resource=check)

                if bom.has_component(component=component):
                    component = bom.get_component_by_purl(  # type:ignore[assignment]  # the previous line checks, if exists
                        purl=component.purl
                    )

                if check.guideline:
                    vulnerability = Vulnerability(
                        id=check.check_id,
                        source_name="checkov",
                        description=f"Resource: {check.resource}. {check.check_name}",
                        affects_targets=[BomTarget(ref=component.bom_ref.value)],
                        advisories=[VulnerabilityAdvisory(url=XsUri(check.guideline))],
                    )
                else:
                    vulnerability = Vulnerability(
                        id=check.check_id,
                        source_name="checkov",
                        description=f"Resource: {check.resource}. {check.check_name}",
                        affects_targets=[BomTarget(ref=component.bom_ref.value)],
                    )

                component.add_vulnerability(vulnerability)
                bom.components.add(component)

            for resource in report.extra_resources:
                component = self.create_component(check_type=report.check_type, resource=resource)

                if not bom.has_component(component=component):
                    bom.components.add(component)

        return bom

    def create_component(self, check_type: str, resource: Record | ExtraResource) -> Component:
        """ Creates a component

        Ex.
        <component bom-ref="pkg:terraform/main.tf/aws_s3_bucket.example@sha1:92911b13224706178dded562c18d281b22bf391a" type="file">
          <name>aws_s3_bucket.example</name>
          <version>sha1:92911b13224706178dded562c18d281b22bf391a</version>
          <hashes>
            <hash alg="SHA-1">92911b13224706178dded562c18d281b22bf391a</hash>
          </hashes>
          <purl>pkg:terraform/main.tf/aws_s3_bucket.example@sha1:92911b13224706178dded562c18d281b22bf391a</purl>
        </component>
        """

        sha1_hash = sha1sum(filename=resource.file_abs_path)
        purl = PackageURL(
            type=check_type,
            namespace=resource.file_path,
            name=resource.resource,
            version=f"sha1:{sha1_hash}",
        )
        component = Component(
            bom_ref=str(purl),
            name=resource.resource,
            version=f"sha1:{sha1_hash}",
            hashes=[
                HashType(
                    algorithm=HashAlgorithm.SHA_1,
                    hash_value=sha1_hash,
                )
            ],
            component_type=ComponentType.APPLICATION,
            purl=purl,
        )

        return component

    def get_xml_output(self) -> str:
        schema_version = CYCLONE_SCHEMA_VERSION.get(
            os.getenv("CHECKOV_CYCLONEDX_SCHEMA_VERSION", ""), DEFAULT_CYCLONE_SCHEMA_VERSION
        )
        output = get_instance(bom=self.bom, schema_version=schema_version).output_as_string()

        return output



from __future__ import annotations

import logging
import os
from typing import TYPE_CHECKING, Callable

from checkov.common.output.record import Record
from checkov.common.output.report import Report, CheckType
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths
from checkov.common.util.dockerfile import is_docker_file
from checkov.common.typing import _CheckResult
from checkov.dockerfile.parser import parse, collect_skipped_checks
from checkov.dockerfile.registry import registry
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from checkov.common.parsers.node import DictNode
    from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck


class Runner(BaseRunner):
    check_type = CheckType.DOCKERFILE  # noqa: CCE003  # a static attribute

    def should_scan_file(self, filename: str) -> bool:
        return is_docker_file(os.path.basename(filename))

    def run(
        self,
        root_folder: str | None = None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True,
    ) -> Report:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)
        files_list = []
        filepath_fn = None
        if external_checks_dir:
            for directory in external_checks_dir:
                registry.load_external_checks(directory)

        if files:
            files_list = [file for file in files if is_docker_file(os.path.basename(file))]

        if root_folder:
            filepath_fn = lambda f: f'/{os.path.relpath(f, os.path.commonprefix((root_folder, f)))}'
            for root, d_names, f_names in os.walk(root_folder):
                filter_ignored_paths(root, d_names, runner_filter.excluded_paths)
                filter_ignored_paths(root, f_names, runner_filter.excluded_paths)
                for file in f_names:
                    if is_docker_file(file):
                        file_path = os.path.join(root, file)
                        files_list.append(file_path)

        definitions, definitions_raw = get_files_definitions(files_list, filepath_fn)
        self.pbar.initiate(len(definitions))
        for docker_file_path in definitions.keys():
            self.pbar.set_additional_data({'Current File Scanned': os.path.relpath(docker_file_path, root_folder)})
            # There are a few cases here. If -f was used, there could be a leading / because it's an absolute path,
            # or there will be no leading slash; root_folder will always be none.
            # If -d is used, root_folder will be the value given, and -f will start with a / (hardcoded above).
            # The goal here is simply to get a valid path to the file (which docker_file_path does not always give).
            if docker_file_path[0] == '/':
                path_to_convert = (root_folder + docker_file_path) if root_folder else docker_file_path
            else:
                path_to_convert = (os.path.join(root_folder, docker_file_path)) if root_folder else docker_file_path

            file_abs_path = os.path.abspath(path_to_convert)
            report.add_resource(file_abs_path)
            skipped_checks = collect_skipped_checks(definitions[docker_file_path])
            instructions = definitions[docker_file_path]

            results = registry.scan(docker_file_path, instructions, skipped_checks,
                                    runner_filter)

            for check, check_result in results.items():
                result_configuration = check_result['results_configuration']
                startline = 0
                endline = len(definitions_raw[docker_file_path]) - 1
                result_instruction = ""
                if result_configuration:
                    if isinstance(result_configuration, list):
                        for res in result_configuration:
                            startline = res['startline']
                            endline = res['endline']
                            result_instruction = res["instruction"]
                            self.build_record(report,
                                              definitions_raw,
                                              docker_file_path,
                                              file_abs_path,
                                              check,
                                              check_result,
                                              startline,
                                              endline,
                                              result_instruction)
                    else:
                        startline = result_configuration['startline']
                        endline = result_configuration['endline']
                        result_instruction = result_configuration["instruction"]
                        self.build_record(report,
                                          definitions_raw,
                                          docker_file_path,
                                          file_abs_path,
                                          check,
                                          check_result,
                                          startline,
                                          endline,
                                          result_instruction)
                else:
                    self.build_record(report,
                                      definitions_raw,
                                      docker_file_path,
                                      file_abs_path,
                                      check,
                                      check_result,
                                      startline,
                                      endline,
                                      result_instruction)
            self.pbar.update()
        self.pbar.close()
        return report

    def calc_record_codeblock(
        self,
        codeblock: list[tuple[int, str]],
        definitions_raw: dict[str, list[str]],
        docker_file_path: str,
        endline: int,
        startline: int,
    ) -> None:
        for line in range(startline, endline + 1):
            codeblock.append((line + 1, definitions_raw[docker_file_path][line]))

    def build_record(
        self,
        report: Report,
        definitions_raw: dict[str, list[str]],
        docker_file_path: str,
        file_abs_path: str,
        check: BaseDockerfileCheck,
        check_result: _CheckResult,
        startline: int,
        endline: int,
        result_instruction: str,
    ) -> None:
        codeblock: list[tuple[int, str]] = []
        self.calc_record_codeblock(codeblock, definitions_raw, docker_file_path, endline, startline)
        record = Record(
            check_id=check.id,
            bc_check_id=check.bc_id,
            check_name=check.name,
            check_result=check_result,
            code_block=codeblock,
            file_path=docker_file_path,
            file_line_range=[startline + 1, endline + 1],
            resource=f"{docker_file_path}.{result_instruction}",
            evaluations=None,
            check_class=check.__class__.__module__,
            file_abs_path=file_abs_path,
            entity_tags=None,
            severity=check.severity,
        )
        record.set_guideline(check.guideline)
        report.add_record(record=record)


def get_files_definitions(
    files: list[str], filepath_fn: Callable[[str], str] | None = None
) -> tuple[dict[str, DictNode], dict[str, list[str]]]:
    def _parse_file(file: str) -> tuple[str, tuple[dict[str, list[dict[str, int | str]]], list[str]] | None]:
        try:
            return file, parse(file)
        except TypeError:
            logging.info(f'Dockerfile skipping {file} as it is not a valid dockerfile template')
            return file, None
        except UnicodeDecodeError:
            logging.info(f'Dockerfile skipping {file} as it can\'t be read as text file')
            return file, None

    results = parallel_runner.run_function(_parse_file, files)
    definitions = {}
    definitions_raw = {}
    for file, result in results:
        if result:
            path = filepath_fn(file) if filepath_fn else file
            definitions[path], definitions_raw[path] = result

    return definitions, definitions_raw
